<?php

namespace Drupal\pagedesigner\Plugin\rest\resource;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Service\ElementHandler;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\ui_patterns\UiPatternsManager;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\UnprocessableEntityHttpException;

/**
 * Provides a resource to get view modes by entity and bundle.
 *
 * @RestResource(
 *   id = "pagedesigner_element",
 *   label = @Translation("Pagedesigner element resource"),
 *   uri_paths = {
 *     "canonical" = "/pagedesigner/element/{id}",
 *     "create" = "/pagedesigner/element",
 *   }
 * )
 */
class ElementResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The handler manager.
   *
   * @var \Drupal\pagedesigner\Service\ElementHandler
   */
  protected $elementHandler;

  /**
   * The handler manager.
   *
   * @var \Drupal\ui_patterns\UiPatternsManager
   */
  protected $patternManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager = NULL;

  /**
   * Constructs a new ElementResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   * @param \Drupal\pagedesigner\Service\ElementHandler $element_handler
   *   The element handler.
   * @param \Drupal\ui_patterns\UiPatternsManager $pattern_manager
   *   The pattern manager.
   * @param \Drupal\Core\Language\LanguageManager $language_manager
   *   The language manager.
   */
  public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        array $serializer_formats,
        LoggerInterface $logger,
        AccountProxyInterface $current_user,
        ElementHandler $element_handler,
        UiPatternsManager $pattern_manager,
        LanguageManager $language_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);

    $this->currentUser = $current_user;
    $this->elementHandler = $element_handler;
    $this->patternManager = $pattern_manager;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->getParameter('serializer.formats'),
          $container->get('logger.factory')->get('pagedesigner'),
          $container->get('current_user'),
          $container->get('pagedesigner.service.element_handler'),
          $container->get('plugin.manager.ui_patterns'),
          $container->get('language_manager')
      );
  }

  /**
   * Responds to GET requests.
   *
   * @param int $id
   *   ID of the entity.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The HTTP response object.
   */
  public function get($id = NULL) {

    $build = [];
    if (!is_numeric($id)) {
      throw new BadRequestHttpException('The entity key must be numeric.');
    }
    $entity = Element::load($id);
    if ($entity == NULL) {
      throw new NotFoundHttpException('The entity does not exist.');
    }

    if ($entity->isTranslatable()) {
      $language = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
      if (!$entity->hasTranslation($language)) {
        throw new UnprocessableEntityHttpException('The entity does not exist in the given language.');
      }
      $entity = $entity->getTranslation($language);
    }

    if ($entity != NULL) {
      $build = $this->elementHandler->serialize($entity);
    }

    $response = new ResourceResponse($build, 200);
    $response->addCacheableDependency(['#cache' => ['max-age' => 0]]);
    return $response;

  }

  /**
   * Responds to POST requests.
   *
   *   The request.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\BadRequestHttpException
   *   If the request is malformed.
   */
  public function post(Request $request) {
    $requestContent = Json::decode($request->getContent());

    if (empty($requestContent['pattern'])) {
      throw new BadRequestHttpException('The pattern key is mandatory for the post requests.');
    }
    if (empty($requestContent['container'])) {
      throw new BadRequestHttpException('The container key is mandatory for the post requests.');
    }
    if (empty($requestContent['entity'])) {
      throw new BadRequestHttpException('The entity key is mandatory for the post requests.');
    }
    if (empty($requestContent['parent']) && $requestContent['parent'] != NULL) {
      throw new BadRequestHttpException('The parent key is mandatory for the post requests.');
    }
    if (!$this->currentUser->hasPermission('edit pagedesigner element entities')) {
      throw new AccessDeniedHttpException('You are not allowed to create pagedesigner content');
    }

    $type = NULL;
    $patternDefinition = NULL;
    $patternDefinitions = $this->patternManager->getDefinitions();
    $this->elementHandler->collectPatterns($patternDefinitions);
    foreach ($patternDefinitions as $plugin_id => $definition) {
      if ($plugin_id == $requestContent['pattern']) {
        $patternDefinition = $definition;
      }
    }
    if ($patternDefinition === NULL) {
      throw new UnprocessableEntityHttpException('The given pattern is not registered.');
    }
    $type = $patternDefinition->getAdditional()['type'];
    if (empty($type)) {
      throw new UnprocessableEntityHttpException('No type given in pattern definition.');
    }
    $isDesignerPattern = FALSE;
    if (isset($patternDefinition->getAdditional()['designer'])) {
      $isDesignerPattern = !empty($patternDefinition->getAdditional()['designer']) || $patternDefinition->getAdditional()['designer'] == 1;
    }
    $designerPermission = $this->currentUser->hasPermission('access pagedesigner designer patterns');
    // Check if the pattern is for designer only.
    if ($isDesignerPattern && !$designerPermission) {
      throw new AccessDeniedHttpException('You are not allowed to create designer elements.');
    }
    $entity = $this->elementHandler->generate($patternDefinition, $requestContent);
    $build = $this->elementHandler->describe($entity);
    return new ModifiedResourceResponse($build, 200);
  }

  /**
   * Responds to PATCH requests.
   *
   * @param int $id
   *   The entity key.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function patch($id, Request $request) {
    $requestContent = Json::decode($request->getContent());

    if (!is_numeric($id)) {
      throw new BadRequestHttpException('The entity key must be numeric.');
    }

    if (!$this->currentUser->hasPermission('edit pagedesigner element entities')) {
      throw new AccessDeniedHttpException('You are not allowed to edit pagedesigner content');
    }

    $entity = Element::load($id);
    if ($entity == NULL) {
      throw new NotFoundHttpException('The entity does not exist.');
    }
    $language = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    if (!$entity->hasTranslation($language)) {
      throw new UnprocessableEntityHttpException('The entity does not exist in the given language.');
    }
    $entity = $entity->getTranslation($language);
    $this->elementHandler->patch($entity, $requestContent);
    $build = $this->elementHandler->serialize($entity);
    return new ModifiedResourceResponse($build, 200);
  }

  /**
   * Responds to DELETE requests.
   *
   * @param int $id
   *   ID of the entity.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function delete($id) {
    if (!is_numeric($id)) {
      throw new BadRequestHttpException('The entity key must be numeric.');
    }
    if (!$this->currentUser->hasPermission('edit pagedesigner element entities')) {
      throw new AccessDeniedHttpException('You are not allowed to delete pagedesigner content');
    }

    $entity = Element::load($id);
    if ($entity == NULL) {
      throw new NotFoundHttpException('The entity does not exist.');
    }
    $language = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    if (!$entity->hasTranslation($language)) {
      throw new UnprocessableEntityHttpException('The entity does not exist in the given language.');
    }
    $entity = $entity->getTranslation($language);
    $build = $this->elementHandler->delete($entity);
    return new ModifiedResourceResponse($build, 200);
  }

}
