<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\TempStore\SharedTempStoreFactory;

/**
 * Node locking class.
 *
 * Provides an interface to create and release locks on page nodes.
 */
class Locker {
  /**
   * The storage for the lock.
   *
   * @var \Drupal\user\SharedTempStore
   */
  protected $store = NULL;

  /**
   * The page key for the lock.
   *
   * @var string
   */
  protected $key = NULL;

  /**
   * Creates a new locker instance.
   *
   *   The shared temp store factory.
   */
  public function __construct(SharedTempStoreFactory $shared_tempstore) {
    $this->store = $shared_tempstore->get('pagedesigner.lock');
  }

  /**
   * Set the entity to create locks for.
   *
   *   The entity.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function setEntity(ContentEntityInterface $entity) {
    $this->key = 'lock:' . $entity->id() . ':' . $entity->language()->getId();
    return $this;
  }

  /**
   * Return the metda data of the current lock.
   *
   * @return object
   *   The meta data.
   */
  public function getMetadata() {
    return $this->store->getMetadata($this->key);
  }

  /**
   * Return the tab identifier of the current lock.
   *
   * @return string
   *   The tab identifier.
   */
  public function getIdentifier() {
    $identifier = $this->store->get($this->key);
    return $identifier;
  }

  /**
   * Check whether current user has a lock, false otherwise.
   *
   * @return bool
   *   Whether there is a lock.
   */
  public function hasLock() {
    if ($this->store->getIfOwner($this->key) !== NULL) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Acquires a lock for the current user using the page information.
   *
   * If an identifier is provided, it must match the existing lock.
   *
   * @param string $identifier
   *   The tab identifier.
   *
   * @return bool
   *   True if lock has been acquired, false otherwise.
   */
  public function acquire($identifier = NULL) {
    // Create if key does not exist or update the lock if user is owner.
    if ($this->createLock($identifier)) {
      return TRUE;
    }
    // Key exists, but user is not owner
    // Check if lock has expired to delete and create a new one.
    $data = $this->store->get($this->key);
    if ($data !== NULL && $this->store->getMetadata($this->key)->getUpdated() < time() - 900) {
      $this->store->delete($this->key);
      return $this->createLock($identifier);
    }

    // Valid lock of other user found.
    return FALSE;
  }

  /**
   * Release the lock of the current user.
   *
   * If an identifier is provided, lock must match identifier.
   *
   * @param string $identifier
   *   The tab identifier.
   *
   * @return bool
   *   Whether the lock has been released.
   */
  public function release($identifier = NULL) {
    $data = $this->store->getIfOwner($this->key);
    if ($data !== NULL) {
      if ($identifier !== NULL) {
        if ($data == $identifier) {
          return $this->store->deleteIfOwner($this->key);
        }
      }
      else {
        return $this->store->deleteIfOwner($this->key);
      }
    }
    return FALSE;
  }

  /**
   * Helper function to create locks.
   *
   * @param string $identifier
   *   The tab identifier.
   *
   * @return bool
   *   True if lock has been acquired, false otherwise.
   */
  protected function createLock(string $identifier = NULL) {
    $lockIdentifier = $this->store->getIfOwner($this->key);
    if ($lockIdentifier !== NULL && $lockIdentifier !== $identifier) {
      return FALSE;
    }
    if (!$this->store->setIfOwner($this->key, $identifier)) {
      return $this->store->setIfNotExists($this->key, $identifier);
    }
    return TRUE;
  }

}
